@@ -6,6 +6,7 @@
 import json
 import csv
 from typing import List, Tuple, Dict
+import re
 
 import numpy as np
 
@@ -35,6 +36,61 @@ def check_filenames(filenames: List[str]):
         raise ValueError(f'Invalid filename(s): {invalid_filenames}')
 
 
+def check_embeddings(path: str):
+    """Raises an error if the embeddings csv file has not the correct format
+    
+    Use this check whenever you want to upload an embedding to the Lightly 
+    Platform.
+    This method only checks whether the header row matches the specs:
+    https://docs.lightly.ai/getting_started/command_line_tool.html#id1
+
+    Args:
+        path:
+            Path to the embedding csv file
+
+    Raises:
+        RuntimeError
+    """
+    header = []
+    with open(path, 'r', newline='') as csv_file:
+        reader = csv.reader(csv_file, delimiter=',')
+        header: List[str] = next(reader)
+
+        # check for whitespace in the header (we don't allow this)
+        if any(x != x.strip() for x in header):
+            raise RuntimeError(
+                'Embeddings csv file must not contain whitespaces.'
+                )
+
+        # first col is `filenames`
+        if header[0] != 'filenames':
+            raise RuntimeError(
+                f'Embeddings csv file must start with `filenames` '
+                f'column but had {header[0]} instead.'
+                )
+
+        # `labels` exists
+        try:
+            header_labels_idx = header.index('labels')
+        except ValueError:
+            raise RuntimeError(f'Embeddings csv file has no `labels` column.')
+
+        # cols between first and `labels` are `embedding_x`
+        for embedding_header in header[1:header_labels_idx]:
+            if not re.match(r'embedding_\d+', embedding_header):
+                raise RuntimeError(
+                    f'Embeddings csv file must have `embedding_x` columns but '
+                    f'found {embedding_header} instead.'
+                    )
+        
+        # check for empty rows in the body of the csv file
+        for i, row in enumerate(reader):
+            if len(row) == 0:
+                raise RuntimeError(
+                    f'Embeddings csv file must not have empty rows. '
+                    f'Found empty row on line {i}.'
+                    )
+
 def save_embeddings(path: str,
                     embeddings: np.ndarray,
                     labels: List[int],
@@ -105,6 +161,8 @@ def load_embeddings(path: str):
         >>>     'path/to/my/embeddings.csv')
 
     """
+    check_embeddings(path)
+
     filenames, labels = [], []
     embeddings = []
     with open(path, 'r', newline='') as csv_file:
